==> percona-server-mongodb_4.0.27.orig.tar.gz: percona-server-mongodb-4.0.27-22/src/mongo/db/matcher/schema/expression_internal_schema_max_properties.h <==

/**
 *    Copyright (C) 2018-present MongoDB, Inc.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the Server Side Public License, version 1,
 *    as published by MongoDB, Inc.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    Server Side Public License for more details.
 *
 *    You should have received a copy of the Server Side Public License
 *    along with this program. If not, see
 *    <http://www.mongodb.com/licensing/server-side-public-license>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the Server Side Public License in all respects for
 *    all of the code used other than as permitted herein. If you modify file(s)
 *    with this exception, you may extend this exception to your version of the
 *    file(s), but you are not obligated to do so. If you do not wish to do so,
 *    delete this exception statement from your version. If you delete this
 *    exception statement from all source files in the program, then also delete
 *    it in the license file.
 */

#pragma once

#include "mongo/db/matcher/schema/expression_internal_schema_num_properties.h"

namespace mongo {

/**
 * MatchExpression for $_internalSchemaMaxProperties keyword. Takes an integer
 * argument that indicates the maximum amount of properties in an object.
 */
class InternalSchemaMaxPropertiesMatchExpression final
    : public InternalSchemaNumPropertiesMatchExpression {
public:
    explicit InternalSchemaMaxPropertiesMatchExpression(long long numProperties)
        : InternalSchemaNumPropertiesMatchExpression(MatchType::INTERNAL_SCHEMA_MAX_PROPERTIES,
                                                     numProperties,
                                                     "$_internalSchemaMaxProperties") {}

    bool matches(const MatchableDocument* doc, MatchDetails* details) const final {
        BSONObj obj = doc->toBSON();
        return (obj.nFields() <= numProperties());
    }

    bool matchesSingleElement(const BSONElement& elem,
                              MatchDetails* details = nullptr) const final {
        if (elem.type() != BSONType::Object) {
            return false;
        }
        return (elem.embeddedObject().nFields() <= numProperties());
    }

    virtual std::unique_ptr<MatchExpression> shallowClone() const final {
        auto maxProperties =
            stdx::make_unique<InternalSchemaMaxPropertiesMatchExpression>(numProperties());
        if (getTag()) {
            maxProperties->setTag(getTag()->clone());
        }
        return std::move(maxProperties);
    }
};
}  // namespace mongo

==> percona-server-mongodb_4.4.16.orig.tar.gz: percona-server-mongodb-4.4.16-16/src/mongo/db/matcher/schema/expression_internal_schema_max_properties.h <==
/**
 *    Copyright (C) 2018-present MongoDB, Inc.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the Server Side Public License, version 1,
 *    as published by MongoDB, Inc.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    Server Side Public License for more details.
 *
 *    You should have received a copy of the Server Side Public License
 *    along with this program. If not, see
 *    <http://www.mongodb.com/licensing/server-side-public-license>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the Server Side Public License in all respects for
 *    all of the code used other than as permitted herein. If you modify file(s)
 *    with this exception, you may extend this exception to your version of the
 *    file(s), but you are not obligated to do so. If you do not wish to do so,
 *    delete this exception statement from your version. If you delete this
 *    exception statement from all source files in the program, then also delete
 *    it in the license file.
 */

#pragma once

#include "mongo/db/matcher/schema/expression_internal_schema_num_properties.h"

namespace mongo {

/**
 * MatchExpression for $_internalSchemaMaxProperties keyword. Takes an integer
 * argument that indicates the maximum amount of properties in an object.
 */
class InternalSchemaMaxPropertiesMatchExpression final
    : public InternalSchemaNumPropertiesMatchExpression {
public:
    explicit InternalSchemaMaxPropertiesMatchExpression(long long numProperties)
        : InternalSchemaNumPropertiesMatchExpression(MatchType::INTERNAL_SCHEMA_MAX_PROPERTIES,
                                                     numProperties,
                                                     "$_internalSchemaMaxProperties") {}

    bool matches(const MatchableDocument* doc, MatchDetails* details) const final {
        BSONObj obj = doc->toBSON();
        return (obj.nFields() <= numProperties());
    }

    bool matchesSingleElement(const BSONElement& elem,
                              MatchDetails* details = nullptr) const final {
        if (elem.type() != BSONType::Object) {
            return false;
        }
        return (elem.embeddedObject().nFields() <= numProperties());
    }

    virtual std::unique_ptr<MatchExpression> shallowClone() const final {
        auto maxProperties =
            std::make_unique<InternalSchemaMaxPropertiesMatchExpression>(numProperties());
        if (getTag()) {
            maxProperties->setTag(getTag()->clone());
        }
        return std::move(maxProperties);
    }
};
}  // namespace mongo
